import Flutter
import TXLiteAVSDK_TRTC

public class TencentTRTCCloud: NSObject, FlutterPlugin, TRTCCloudDelegate {
	
	private static var channel: FlutterMethodChannel?;
	private var trtcCloud: TRTCCloud;
	private var txAudioEffectManager: TXAudioEffectManager;
	private var txBeautyManager: TXBeautyManager;
	private var txDeviceManager: TXDeviceManager;
	
	/**
	* 监听器回调的方法名
	*/
	private static let LISTENER_FUNC_NAME = "onListener";
	
	public override init() {
		trtcCloud = TRTCCloud.sharedInstance();
		txAudioEffectManager = trtcCloud.getAudioEffectManager();
		txBeautyManager = trtcCloud.getBeautyManager();
		txDeviceManager = trtcCloud.getDeviceManager();
	};
	
	public static func register(with registrar: FlutterPluginRegistrar) {
		let channel = FlutterMethodChannel(name: "trtcCloudChannel", binaryMessenger: registrar.messenger())
		let instance = TencentTRTCCloud()

		TencentTRTCCloud.channel = channel;
		registrar.addMethodCallDelegate(instance, channel: channel)
		
		// 视图工厂
		let viewFactory = TRTCCloudVideoPlatformViewFactory(message: registrar.messenger());
		
		// 注册界面
		registrar.register(
			viewFactory,
			withId: TRTCCloudVideoPlatformViewFactory.SIGN
		);
	}
	
	private func txf_log_info(file: String = #file,
							  line: Int = #line,
							  function: String = #function,
							  content: String) {
		txf_log_swift(TXE_LOG_INFO, file.cString(using: .utf8), Int32(line), function.cString(using: .utf8), content)
	}

	private func txf_log_error(file: String = #file,
							  line: Int = #line,
							  function: String = #function,
							  content: String) {
		txf_log_swift(TXE_LOG_ERROR, file.cString(using: .utf8), Int32(line), function.cString(using: .utf8), content)
	}
	
	
	private func logFlutterMethodCall(_ call: FlutterMethodCall) {
		let noAgrs = { [weak self] in
			guard let self = self else {return}
			self.txf_log_info(content: "method=\(call.method)|arguments: nil")
		}
		
		if let args = call.arguments as? Dictionary<String, Any> {
			var argsInfo = ""
			for paramter in args.keys {
				if let value = args[paramter] {
					argsInfo.append("\(paramter):\(value), ")
				}
			}
			if argsInfo.count > 0 {
				txf_log_info(content: "method=\(call.method)|arguments: {\(argsInfo)}")
			} else {
				noAgrs()
			}
		} else {
			noAgrs()
		}
	}

	private func logError(call: FlutterMethodCall, errCode: Int, errMsg: String) {
		let noAgrs = { [weak self] in
			guard let self = self else {return}
			self.txf_log_error(content: "method=\(call.method)|arguments=nil|error={errCode: \(errCode), errMsg: \(errMsg)}")
		}
		
		if let args = call.arguments as? Dictionary<String, Any> {
			var argsInfo = ""
			for paramter in args.keys {
				if let value = args[paramter] {
					argsInfo.append("\(paramter):\(value), ")
				}
			}
			if argsInfo.count > 0 {
				txf_log_error(content: "method=\(call.method)|arguments={\(argsInfo)}|error={errCode: \(errCode), errMsg: \(errMsg)}")
			} else {
				noAgrs()
			}
		} else {
			noAgrs()
		}
	}



	
	
	public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
		defer {
			logFlutterMethodCall(call)
		}
	
		switch call.method {
		case "sharedInstance":
			self.sharedInstance(call: call, result: result);
			break;
		case "destroySharedInstance":
			self.destroySharedInstance(call: call, result: result);
			break;
		case "enterRoom":
			self.enterRoom(call: call, result: result);
			break;
		case "exitRoom":
			self.exitRoom(call: call, result: result);
			break;
		case "switchRoom":
			self.switchRoom(call: call, result: result);
			break;
		case "connectOtherRoom":
			self.connectOtherRoom(call: call, result: result);
			break;
		case "disconnectOtherRoom":
			self.disconnectOtherRoom(call: call, result: result);
			break;
		case "switchRole":
			self.switchRole(call: call, result: result);
			break;
		case "setDefaultStreamRecvMode":
			self.setDefaultStreamRecvMode(call: call, result: result);
			break;
		case "stopLocalPreview":
			self.stopLocalPreview(call: call, result: result);
			break;
		case "muteRemoteAudio":
			self.muteRemoteAudio(call: call, result: result);
			break;
		case "muteAllRemoteAudio":
			self.muteAllRemoteAudio(call: call, result: result);
			break;
		case "setRemoteAudioVolume":
			self.setRemoteAudioVolume(call: call, result: result);
			break;
		case "setAudioCaptureVolume":
			self.setAudioCaptureVolume(call: call, result: result);
			break;
		case "getAudioCaptureVolume":
			self.getAudioCaptureVolume(call: call, result: result);
			break;
		case "setAudioPlayoutVolume":
			self.setAudioPlayoutVolume(call: call, result: result);
			break;
		case "getAudioPlayoutVolume":
			self.getAudioPlayoutVolume(call: call, result: result);
			break;
    	case "stopRemoteView":
			self.stopRemoteView(call: call, result: result);
			break;
		case "startLocalAudio":
			self.startLocalAudio(call: call, result: result);
			break;
		case "stopLocalAudio":
			self.stopLocalAudio(call: call, result: result);
			break;
		case "setLocalRenderParams":
			self.setLocalRenderParams(call: call, result: result);
			break;
		case "setRemoteRenderParams":
			self.setRemoteRenderParams(call: call, result: result);
			break;
		case "stopAllRemoteView":
			self.stopAllRemoteView(call: call, result: result);
			break;
		case "muteRemoteVideoStream":
			self.muteRemoteVideoStream(call: call, result: result);
			break;
		case "muteAllRemoteVideoStreams":
			self.muteAllRemoteVideoStreams(call: call, result: result);
			break;
		case "setVideoEncoderParam":
			self.setVideoEncoderParam(call: call, result: result);
			break;
		case "startPublishing":
			self.startPublishing(call: call, result: result);
			break;
		case "stopPublishing":
			self.stopPublishing(call: call, result: result);
			break;
    	case "setMixTranscodingConfig":
			self.setMixTranscodingConfig(call: call, result: result);
			break;
		case "setNetworkQosParam":
			self.setNetworkQosParam(call: call, result: result);
			break;
		case "setVideoEncoderRotation":
			self.setVideoEncoderRotation(call: call, result: result);
			break;
		case "setVideoMuteImage":
			self.setVideoMuteImage(call: call, result: result);
			break;
		case "setVideoEncoderMirror":
			self.setVideoEncoderMirror(call: call, result: result);
			break;
		case "setGSensorMode":
			self.setGSensorMode(call: call, result: result);
			break;
		case "enableEncSmallVideoStream":
			self.enableEncSmallVideoStream(call: call, result: result);
			break;
		case "setRemoteVideoStreamType":
			self.setRemoteVideoStreamType(call: call, result: result);
			break;
		case "snapshotVideo":
      		self.snapshotVideo(call: call, result: result);
            break;
		case "muteLocalAudio":
			self.muteLocalAudio(call: call, result: result);
			break;
		case "muteLocalVideo":
			self.muteLocalVideo(call: call, result: result);
			break;
		case "setAudioRoute":
			self.setAudioRoute(call: call, result: result);
			break;
		case "enableAudioVolumeEvaluation":
			self.enableAudioVolumeEvaluation(call: call, result: result);
			break;
		case "startAudioRecording":
			self.startAudioRecording(call: call, result: result);
			break;
		case "stopAudioRecording":
			self.stopAudioRecording(call: call, result: result);
			break;
		case "setSystemVolumeType":
			self.setSystemVolumeType(call: call, result: result);
			break;
		case "enableAudioEarMonitoring":
			// self.enableAudioEarMonitoring(call: call, result: result);
			break;
		case "switchCamera":
			self.switchCamera(call: call, result: result);
			break;
		case "isFrontCamera":
			self.isFrontCamera(call: call, result: result);
			break;
		case "getCameraZoomMaxRatio":
			self.getCameraZoomMaxRatio(call: call, result: result);
			break;
		case "setCameraZoomRatio":
			self.setCameraZoomRatio(call: call, result: result);
			break;
		case "enableCameraAutoFocus":
			self.enableCameraAutoFocus(call: call, result: result);
			break;
		case "enableCameraTorch":
			self.enableCameraTorch(call: call, result: result);
			break;
		case "setCameraFocusPosition":
			self.setCameraFocusPosition(call: call, result: result);
			break;
		case "isAutoFocusEnabled":
			self.isAutoFocusEnabled(call: call, result: result);
			break;
		case "getBeautyManager":
			self.getBeautyManager(call: call, result: result);
			break;
		case "getDeviceManager":
			self.getBeautyManager(call: call, result: result);
			break;
		case "setWatermark":
			self.setWatermark(call: call, result: result);
			break;
		case "getAudioEffectManager":
			self.getAudioEffectManager(call: call, result: result);
			break;
		case "sendCustomCmdMsg":
            self.sendCustomCmdMsg(call: call, result: result);
            break;
        case "sendSEIMsg":
            self.sendSEIMsg(call: call, result: result);
            break;
		case "startSpeedTest":
			self.startSpeedTest(call: call, result: result);
			break;
		case "stopSpeedTest":
			self.stopSpeedTest(call: call, result: result);
			break;
		case "getSDKVersion":
			self.getSDKVersion(call: call, result: result);
			break;
		case "setLogCompressEnabled":
			self.setLogCompressEnabled(call: call, result: result);
			break;
		case "setLogDirPath":
			self.setLogDirPath(call: call, result: result);
			break;
		case "setLogLevel":
			self.setLogLevel(call: call, result: result);
			break;
		case "setConsoleEnabled":
			self.setConsoleEnabled(call: call, result: result);
			break;
		case "showDebugView":
			self.showDebugView(call: call, result: result);
			break;
		case "setBeautyStyle":
			self.setBeautyStyle(call: call, result: result);
			break;
		case "setBeautyLevel":
			self.setBeautyLevel(call: call, result: result);
			break;
		case "setWhitenessLevel":
			self.setWhitenessLevel(call: call, result: result);
			break;
		case "enableSharpnessEnhancement":
			self.enableSharpnessEnhancement(call: call, result: result);
			break;
		case "setRuddyLevel":
			self.setRuddyLevel(call: call, result: result);
			break;
		case "setFilter":
			self.setFilter(call: call, result: result);
			break;
		case "setFilterStrength":
			self.setFilterStrength(call: call, result: result);
			break;
        case "setEyeScaleLevel":
			self.setEyeScaleLevel(call: call, result: result);
			break;
		case "setFaceSlimLevel":
			self.setFaceSlimLevel(call: call, result: result);
			break;
		case "setFaceVLevel":
			self.setFaceVLevel(call: call, result: result);
			break;
		case "setChinLevel":
			self.setChinLevel(call: call, result: result);
			break;
		case "setFaceShortLevel":
			self.setFaceShortLevel(call: call, result: result);
			break;
		case "setNoseSlimLevel":
			self.setNoseSlimLevel(call: call, result: result);
			break;
		case "setEyeLightenLevel":
			self.setEyeLightenLevel(call: call, result: result);
			break;
		case "setToothWhitenLevel":
			self.setToothWhitenLevel(call: call, result: result);
			break;
		case "setWrinkleRemoveLevel":
			self.setWrinkleRemoveLevel(call: call, result: result);
			break;
		case "setPounchRemoveLevel":
			self.setPounchRemoveLevel(call: call, result: result);
			break;
		case "setSmileLinesRemoveLevel":
			self.setSmileLinesRemoveLevel(call: call, result: result);
			break;
		case "setForeheadLevel":
			self.setForeheadLevel(call: call, result: result);
			break;
		case "setEyeDistanceLevel":
			self.setEyeDistanceLevel(call: call, result: result);
			break;
		case "setEyeAngleLevel":
			self.setEyeAngleLevel(call: call, result: result);
			break;
		case "setMouthShapeLevel":
			self.setMouthShapeLevel(call: call, result: result);
			break;
		case "setNoseWingLevel":
			self.setNoseWingLevel(call: call, result: result);
			break;
		case "setNosePositionLevel":
			self.setNosePositionLevel(call: call, result: result);
			break;
		case "setLipsThicknessLevel":
			self.setLipsThicknessLevel(call: call, result: result);
			break;
		case "setFaceBeautyLevel":
			self.setFaceBeautyLevel(call: call, result: result);
			break;
		case "enableVoiceEarMonitor":
			self.enableVoiceEarMonitor(call: call, result: result);
			break;
		case "setVoiceEarMonitorVolume":
			self.setVoiceEarMonitorVolume(call: call, result: result);
			break;
		case "setVoiceReverbType":
			self.setVoiceReverbType(call: call, result: result);
			break;
		case "setVoiceChangerType":
			self.setVoiceChangerType(call: call, result: result);
			break;
		case "setVoiceCaptureVolume":
			self.setVoiceCaptureVolume(call: call, result: result);
			break;
		case "setMusicObserver":
			// self.setMusicObserver(call: call, result: result);
			break;
		case "startPlayMusic":
			self.startPlayMusic(call: call, result: result);
			break;
		case "stopPlayMusic":
			self.stopPlayMusic(call: call, result: result);
			break;
		case "pausePlayMusic":
			self.pausePlayMusic(call: call, result: result);
			break;
		case "resumePlayMusic":
			self.resumePlayMusic(call: call, result: result);
			break;
		case "setMusicPublishVolume":
			self.setMusicPublishVolume(call: call, result: result);
			break;
		case "setMusicPlayoutVolume":
			self.setMusicPlayoutVolume(call: call, result: result);
			break;
		case "setAllMusicVolume":
			self.setAllMusicVolume(call: call, result: result);
			break;
		case "setMusicPitch":
			self.setMusicPitch(call: call, result: result);
			break;
		case "setMusicSpeedRate":
			self.setMusicSpeedRate(call: call, result: result);
			break;
		case "getMusicCurrentPosInMS":
			self.getMusicCurrentPosInMS(call: call, result: result);
			break;
		case "seekMusicToPosInMS":
			self.seekMusicToPosInMS(call: call, result: result);
			break;
		case "getMusicDurationInMS":
			self.getMusicDurationInMS(call: call, result: result);
			break;
		default:
			logError(call: call, errCode: -1, errMsg: "method not Implemented");
			result(FlutterMethodNotImplemented);
		}
	}

	/**
	* 停止本地视频采集及预览
	*/
	public func stopLocalPreview(call: FlutterMethodCall, result: @escaping FlutterResult) {
		trtcCloud.stopLocalPreview();
		result(nil);
	}

  	/**
    * 停止显示远端视频画面，同时不再拉取该远端用户的视频数据流
	* 调用此接口后，SDK 会停止接收该用户的远程视频流，同时会清理相关的视频显示资源。
    */
	public func stopRemoteView(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let userId = CommonUtils.getParamByKey(call: call, result: result, param: "userId") as? String,
			let streamType = CommonUtils.getParamByKey(call: call, result: result, param: "streamType") as? Int {
			trtcCloud.stopRemoteView(userId, streamType: TRTCVideoStreamType(rawValue: streamType)!);
			result(nil);
		}
	}
	
	/**
	* 显示仪表盘
	*/
	public func showDebugView(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let mode = CommonUtils.getParamByKey(call: call, result: result, param: "mode") as? Int {
			trtcCloud.showDebugView(mode);
			result(nil);
		}
	}
	
	/**
	* 启用或禁用控制台日志打印
	*/
	public func setConsoleEnabled(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enabled = CommonUtils.getParamByKey(call: call, result: result, param: "enabled") as? Bool {
			TRTCCloud.setConsoleEnabled(enabled);
			result(nil);
		}
	}

	/**
	* 启用或禁用 Log 的本地压缩。
	*/
	public func setLogCompressEnabled(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enabled = CommonUtils.getParamByKey(call: call, result: result, param: "enabled") as? Bool {
			TRTCCloud.setLogCompressEnabled(enabled);
			result(nil);
		}
	}

	/**
	* 启用或禁用 Log 的本地压缩。
	*/
	public func setLogDirPath(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let path = CommonUtils.getParamByKey(call: call, result: result, param: "path") as? String {
			TRTCCloud.setLogDirPath(path);
			result(nil);
		}
	}
	
	/**
	* 设置 Log 输出级别
	*/
	public func setLogLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let level = CommonUtils.getParamByKey(call: call, result: result, param: "level") as? Int {
			TRTCCloud.setLogLevel(TRTCLogLevel(rawValue: level)!);
			result(nil);
		}
	}
	
	/**
	* 进入房间
	* 调用接口后，您会收到来自 TRTCCloudListener 中的 onEnterRoom(result) 回调：
	*/
	public func enterRoom(call: FlutterMethodCall, result: @escaping FlutterResult) {
		let param = JsonUtil.getDictionaryFromJSONString(jsonString: (CommonUtils.getParamByKey(call: call, result: result, param: "param") as? String)!);
		let scene = ((call.arguments as! [String: Any])["scene"]) as? Int;

		if let sdkAppId = param["sdkAppId"] as? UInt32,
			 let userId = param["userId"] as? String,
			 let userSig = param["userSig"] as? String,
			 let roomId = param["roomId"] as? UInt32,
			 let role = param["role"] as? Int,
			 let streamId = param["streamId"] as? String,
			 let userDefineRecordId = param["userDefineRecordId"] as? String,
			 let privateMapKey = param["privateMapKey"] as? String,
			 let businessInfo = param["businessInfo"] as? String {
			let params = TRTCParams();
			params.sdkAppId = sdkAppId;
			params.userId = userId;
			params.userSig = userSig;
			params.roomId = roomId;
			params.streamId = streamId;
			params.userDefineRecordId = userDefineRecordId;
			params.privateMapKey = privateMapKey;
			params.bussInfo = businessInfo;
            params.role = TRTCRoleType(rawValue: role)!;
			
			trtcCloud.enterRoom(params, appScene: TRTCAppScene(rawValue: scene!)!);
			result(nil);
		}
	}
	
	/**
	* 离开房间
	*/
	public func exitRoom(call: FlutterMethodCall, result: @escaping FlutterResult) {
		trtcCloud.exitRoom();
		result(nil);
	}
	
	/**
	* 跨房通话
	*/
	public func connectOtherRoom(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let param = CommonUtils.getParamByKey(call: call, result: result, param: "param") as? String {
			trtcCloud.connectOtherRoom(param);
			result(nil);
		}
	}

	/**
	* 退出跨房通话
	*/
	public func disconnectOtherRoom(call: FlutterMethodCall, result: @escaping FlutterResult) {
		trtcCloud.disconnectOtherRoom();
		result(nil);
	}
	
	/**
	* 切换房间
	*/
	public func switchRoom(call: FlutterMethodCall, result: @escaping FlutterResult) {
		let config = JsonUtil.getDictionaryFromJSONString(jsonString: (CommonUtils.getParamByKey(call: call, result: result, param: "config") as? String)!);

		if let userSig = config["userSig"] as? String,
			 let roomId = config["roomId"] as? UInt32,
			 let strRoomId = config["strRoomId"] as? String,
			 let privateMapKey = config["privateMapKey"] as? String {
			let params = TRTCSwitchRoomConfig();
			params.userSig = userSig;
			params.roomId = roomId;
			params.strRoomId = strRoomId;
			params.privateMapKey = privateMapKey;
			
			trtcCloud.switchRoom(params);
			result(nil);
		}
	}

	/**
	* 切换角色，仅适用于直播场景（TRTC_APP_SCENE_LIVE 和 TRTC_APP_SCENE_VOICE_CHATROOM）
	*/
	public func switchRole(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let role = CommonUtils.getParamByKey(call: call, result: result, param: "role") as? Int {
			trtcCloud.switch(TRTCRoleType(rawValue: role)!);
			result(nil);
		}
	}
	
	/**
	* 设置音视频数据接收模式，需要在进房前设置才能生效
	*/
	public func setDefaultStreamRecvMode(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let autoRecvAudio = CommonUtils.getParamByKey(call: call, result: result, param: "autoRecvAudio") as? Bool,
			 let autoRecvVideo = CommonUtils.getParamByKey(call: call, result: result, param: "autoRecvVideo") as? Bool {
			trtcCloud.setDefaultStreamRecvMode(autoRecvAudio, video: autoRecvVideo);
			result(nil);
		}
	}
	
	/**
	* 静音/取消静音指定的远端用户的声音
	*/
	public func muteRemoteAudio(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let userId = CommonUtils.getParamByKey(call: call, result: result, param: "userId") as? String,
			 let mute = CommonUtils.getParamByKey(call: call, result: result, param: "mute") as? Bool {
			trtcCloud.muteRemoteAudio(userId, mute: mute);
			result(nil);
		}
	}
	
	/**
	* 静音/取消静音所有用户的声音
	*/
	public func muteAllRemoteAudio(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let mute = CommonUtils.getParamByKey(call: call, result: result, param: "mute") as? Bool {
			trtcCloud.muteAllRemoteAudio(mute);
			result(nil);
		}
	}
	
	/**
	* 设置采集音量
	*/
	public func setAudioCaptureVolume(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let volume = CommonUtils.getParamByKey(call: call, result: result, param: "volume") as? Int {
			trtcCloud.setAudioCaptureVolume(volume);
			result(nil);
		}
	}
	
	/**
	* 设置某个远程用户的播放音量
	*/
	public func setRemoteAudioVolume(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let userId = CommonUtils.getParamByKey(call: call, result: result, param: "userId") as? String,
			 let volume = CommonUtils.getParamByKey(call: call, result: result, param: "volume") as? Int32 {
			trtcCloud.setRemoteAudioVolume(userId, volume: volume);
			result(nil);
		}
	}
	
	/**
	* 设置播放音量
	*/
	public func setAudioPlayoutVolume(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let volume = CommonUtils.getParamByKey(call: call, result: result, param: "volume") as? Int {
			trtcCloud.setAudioPlayoutVolume(volume);
			result(nil);
		}
	}
	
	/**
	* 获取采集音量
	*/
	public func getAudioCaptureVolume(call: FlutterMethodCall, result: @escaping FlutterResult) {
		let volume = trtcCloud.getAudioCaptureVolume();
		result(volume);
	}
	
	/**
	* 获取播放音量
	*/
	public func getAudioPlayoutVolume(call: FlutterMethodCall, result: @escaping FlutterResult) {
		let volume = trtcCloud.getAudioPlayoutVolume();
		result(volume);
	}
	
	/**
	* 开启本地音频的采集和上行
	*/
	public func startLocalAudio(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let quality = CommonUtils.getParamByKey(call: call, result: result, param: "quality") as? Int {
			trtcCloud.startLocalAudio(TRTCAudioQuality(rawValue: quality)!);
			result(nil);
		}
	}
	
	/**
	* 关闭本地音频的采集和上行
	*/
	public func stopLocalAudio(call: FlutterMethodCall, result: @escaping FlutterResult) {
		trtcCloud.stopLocalAudio();
		result(nil);
	}

	

	/**
	* 本地视频渲染设置
	*/
	public func setLocalRenderParams(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let param = CommonUtils.getParamByKey(call: call, result: result, param: "param") as? String {
			let dict = JsonUtil.getDictionaryFromJSONString(jsonString: param);
			let data = TRTCRenderParams();
			if dict["rotation"] != nil {
				data.rotation = TRTCVideoRotation(rawValue: dict["rotation"] as! Int)!;
			}
			if dict["fillMode"] != nil {
				data.fillMode = TRTCVideoFillMode(rawValue: dict["fillMode"] as! Int)!;
			}
			if dict["mirrorType"] != nil {
				data.mirrorType = TRTCVideoMirrorType(rawValue: dict["mirrorType"] as! UInt)!;
			}
			trtcCloud.setLocalRenderParams(data);
			result(nil);
		}
	}

	/**
	* 远程视频渲染设置
	*/
	public func setRemoteRenderParams(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let userId = CommonUtils.getParamByKey(call: call, result: result, param: "userId") as? String,
			let streamType = CommonUtils.getParamByKey(call: call, result: result, param: "streamType") as? Int,
			let param = CommonUtils.getParamByKey(call: call, result: result, param: "param") as? String {
			let dict = JsonUtil.getDictionaryFromJSONString(jsonString: param);
			let data = TRTCRenderParams();
			if dict["rotation"] != nil {
				data.rotation = TRTCVideoRotation(rawValue: dict["rotation"] as! Int)!;
			}
			if dict["fillMode"] != nil {
				data.fillMode = TRTCVideoFillMode(rawValue: dict["fillMode"] as! Int)!;
			}
			if dict["mirrorType"] != nil {
				data.mirrorType = TRTCVideoMirrorType(rawValue: dict["mirrorType"] as! UInt)!;
			}
			trtcCloud.setRemoteRenderParams(userId, streamType: TRTCVideoStreamType(rawValue: streamType)!,  params: data);
			result(nil);
		}
	}
	
	/**
	* 停止显示所有远端视频画面，同时不再拉取远端用户的视频数据流
	*/
	public func stopAllRemoteView(call: FlutterMethodCall, result: @escaping FlutterResult) {
		trtcCloud.stopAllRemoteView();
		result(nil);
	}
	
	/**
	* 暂停/恢复接收指定的远端视频流
	*/
	public func muteRemoteVideoStream(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let userId = CommonUtils.getParamByKey(call: call, result: result, param: "userId") as? String,
			 let mute = CommonUtils.getParamByKey(call: call, result: result, param: "mute") as? Bool {
			trtcCloud.muteRemoteVideoStream(userId, mute: mute);
			result(nil);
		}
	}
	
	/**
	* 暂停/恢复接收所有远端视频流
	*/
	public func muteAllRemoteVideoStreams(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let mute = CommonUtils.getParamByKey(call: call, result: result, param: "mute") as? Bool {
			trtcCloud.muteAllRemoteVideoStreams(mute);
			result(nil);
		}
	}
	
	/**
	* 设置视频编码器相关参数
	*/
	public func setVideoEncoderParam(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let param = CommonUtils.getParamByKey(call: call, result: result, param: "param") as? String {
			let dict = JsonUtil.getDictionaryFromJSONString(jsonString: param);
			let data = TRTCVideoEncParam();
			if dict["videoBitrate"] != nil {
				data.videoBitrate = dict["videoBitrate"] as! Int32;
			}
			if dict["videoResolution"] != nil {
				data.videoResolution = TRTCVideoResolution(rawValue: dict["videoResolution"] as! Int)!;
			}
			if dict["videoResolutionMode"] != nil {
				data.resMode = TRTCVideoResolutionMode(rawValue: dict["videoResolutionMode"] as! Int)!;
			}
			if dict["videoFps"] != nil {
				data.videoFps = dict["videoFps"] as! Int32;
			}
			trtcCloud.setVideoEncoderParam(data);
			result(nil);
		}
	}

	/**
	* 开始向腾讯云的直播 CDN 推流
	*/
	public func startPublishing(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let streamType = CommonUtils.getParamByKey(call: call, result: result, param: "streamType") as? Int,
			let streamId = CommonUtils.getParamByKey(call: call, result: result, param: "streamId") as? String {
            trtcCloud.startPublishing(streamId, type: TRTCVideoStreamType(rawValue: streamType)!);
			result(nil);
			
		}
	}
	
	/**
	* 停止向腾讯云的直播 CDN 推流
	*/
	public func stopPublishing(call: FlutterMethodCall, result: @escaping FlutterResult) {
		trtcCloud.stopPublishing();
		result(nil);
	}

  	/**
	* 设置云端的混流转码参数
	*/
	public func setMixTranscodingConfig(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let param = CommonUtils.getParamByKey(call: call, result: result, param: "config") as? String {
            let dict = JsonUtil.getDictionaryFromJSONString(jsonString: param);
            let backgroundImage = dict["backgroundImage"] as? String;
            let streamId = dict["streamId"] as? String;

			if let appId = dict["appId"] as? Int32,
				let bizId = dict["bizId"] as? Int32,
				let videoWidth = dict["videoWidth"] as? Int32,
				let mode = dict["mode"] as? Int,
				let videoHeight = dict["videoHeight"] as? Int32,
				let videoFramerate = dict["videoFramerate"] as? Int32,
				let videoGOP = dict["videoGOP"] as? Int32,
                let backgroundColor = dict["backgroundColor"] as? Int32,
				let videoBitrate = dict["videoBitrate"] as? Int32,
				let audioBitrate = dict["audioBitrate"] as? Int32,
				let audioSampleRate = dict["audioSampleRate"] as? Int32,
				let audioChannels = dict["audioChannels"] as? Int32,
				let mixUsers = dict["mixUsers"] as? Array<AnyObject> {

				let config = TRTCTranscodingConfig();
				var users: [TRTCMixUser] = [];

				config.appId = appId;
				config.bizId = bizId;
				config.videoWidth = videoWidth;
				config.mode = TRTCTranscodingConfigMode(rawValue: mode)!;
				config.videoHeight = videoHeight;
				config.videoFramerate = videoFramerate;
				config.videoGOP = videoGOP;
                config.backgroundImage = backgroundImage;
                config.backgroundColor = backgroundColor;
				config.videoBitrate = videoBitrate;
				config.audioBitrate = audioBitrate;
				config.audioSampleRate = audioSampleRate;
				config.audioChannels = audioChannels;
				config.streamId = streamId;

				for item in mixUsers {
					let user = TRTCMixUser();
					user.userId = item["userId"] as! String;
					user.roomID = item["roomId"] as? String;
					user.rect = CGRect(x: item["x"] as! Int, y: item["y"] as! Int, width: item["width"] as! Int, height: item["height"] as! Int);
					user.zOrder = item["zOrder"] as! Int32;
					user.streamType = TRTCVideoStreamType(rawValue: item["streamType"] as! Int)!;
					user.pureAudio = false;
					users.append(user);
				}

				config.mixUsers = users;
				trtcCloud.setMix(config);
						
				result(nil);
			}
		}
	}
	
	/**
	* 设置网络流控相关参数
	*/
	public func setNetworkQosParam(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let param = CommonUtils.getParamByKey(call: call, result: result, param: "param") as? String {
			let dict = JsonUtil.getDictionaryFromJSONString(jsonString: param);
			let param = TRTCNetworkQosParam();

			if !(dict["preference"] is NSNull) &&  dict["preference"] != nil {
				param.preference = TRTCVideoQosPreference(rawValue: dict["preference"] as! Int)!;
			}
			if !(dict["controlMode"] is NSNull) &&  dict["controlMode"] != nil {
				param.controlMode = TRTCQosControlMode(rawValue: dict["controlMode"] as! Int)!;
			}

			trtcCloud.setNetworkQosParam(param);
			result(nil);
		}
	}

	/**
	* 设置暂停推送本地视频时要推送的图片
	*/
	public func setVideoMuteImage(call: FlutterMethodCall, result: @escaping FlutterResult) {
		let imageUrl = CommonUtils.getParamByKeyCanBeNull(call: call, result: result, param: "imageUrl") as? String;
		if 	let fps = CommonUtils.getParamByKey(call: call, result: result, param: "fps") as? Int,
			let type = CommonUtils.getParamByKey(call: call, result: result, param: "type") as? String {
			if(imageUrl == nil) {
				trtcCloud.setVideoMuteImage(nil, fps: fps);
			} else {
				if type == "local" {
                    let img = UIImage(contentsOfFile: imageUrl!)!;
					trtcCloud.setVideoMuteImage(img, fps: fps);
				} else {
					let queue = DispatchQueue(label: "setVideoMuteImage")
					queue.async {
                        let url: NSURL = NSURL(string: imageUrl!)!
						let data: NSData = NSData(contentsOf: url as URL)!
						let img = UIImage(data: data as Data, scale: 1)!
						self.trtcCloud.setVideoMuteImage(img, fps: fps);
					}
				}
			}
			result(nil);
		}
	}
	
	/**
	* 设置视频编码输出的画面方向，即设置远端用户观看到的和服务器录制的画面方向
	*/
	public func setVideoEncoderRotation(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let rotation = CommonUtils.getParamByKey(call: call, result: result, param: "rotation") as? Int {
			trtcCloud.setVideoEncoderRotation(TRTCVideoRotation(rawValue: rotation)!);
			result(nil);
		}
	}
	
	/**
	* 设置编码器输出的画面镜像模式
	*/
	public func setVideoEncoderMirror(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let mirror = CommonUtils.getParamByKey(call: call, result: result, param: "mirror") as? Bool {
			trtcCloud.setVideoEncoderMirror(mirror);
			result(nil);
		}
	}
	
	/**
	* 设置重力感应的适应模式
	*/
	public func setGSensorMode(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let mode = CommonUtils.getParamByKey(call: call, result: result, param: "mode") as? Int {
			trtcCloud.setGSensorMode(TRTCGSensorMode(rawValue: mode)!);
			result(nil);
		}
	}
	
	/**
	* 开启大小画面双路编码模式
	*/
	public func enableEncSmallVideoStream(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Bool,
			let smallVideoEncParam = CommonUtils.getParamByKey(call: call, result: result, param: "smallVideoEncParam") as? String {
			let dict = JsonUtil.getDictionaryFromJSONString(jsonString: smallVideoEncParam);
			let data = TRTCVideoEncParam();
			if !(dict["videoBitrate"] is NSNull) &&  dict["videoBitrate"] != nil {
				data.videoBitrate = dict["videoBitrate"] as! Int32;
			}
			if !(dict["videoResolution"] is NSNull)  &&  dict["videoResolution"] != nil  {
				data.videoResolution = TRTCVideoResolution(rawValue: dict["videoResolution"] as! Int)!;
			}
			if !(dict["videoResolutionMode"] is NSNull)  &&  dict["videoResolutionMode"] != nil  {
				data.resMode = TRTCVideoResolutionMode(rawValue: dict["videoResolutionMode"] as! Int)!;
			}
			if !(dict["videoFps"] is NSNull)  &&  dict["videoFps"] != nil  {
				data.videoFps = dict["videoFps"] as! Int32;
			}

			let ret = trtcCloud.enableEncSmallVideoStream(enable, withQuality: data);
			result(ret);
		}
	}
	
	/**
	* 选定观看指定 uid 的大画面或小画面
	*/
	public func setRemoteVideoStreamType(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let userId = CommonUtils.getParamByKey(call: call, result: result, param: "userId") as? String,
			 let streamType = CommonUtils.getParamByKey(call: call, result: result, param: "streamType") as? Int {
			trtcCloud.setRemoteVideoStreamType(userId, type: TRTCVideoStreamType(rawValue: streamType)!);
			result(nil);
		}
	}

  /**
  * 视频画面截图
  */
  public func snapshotVideo(call: FlutterMethodCall, result: @escaping FlutterResult) {
	let userId = CommonUtils.getParamByKey(call: call, result: result, param: "userId") as? String;
    if let streamType = CommonUtils.getParamByKey(call: call, result: result, param: "streamType") as? Int,
		let path = CommonUtils.getParamByKey(call: call, result: result, param: "path") as? String {
		
		trtcCloud.snapshotVideo(userId, type: TRTCVideoStreamType(rawValue: streamType)!, completionBlock: {
        (image) -> Void in
		
		let data: Data
		let url = URL(fileURLWithPath: "path")
			
		if path.hasSuffix(".png") {
			data = (image?.pngData())!
		} else {
			data = (image?.jpegData(compressionQuality: CGFloat(1)))!
		}

		do {
			try data.write(to: url)
			self.invokeListener(type: ListenerType.onSnapshotComplete, params: ["errCode": 0, "path": path]);
		} catch {
			self.logError(call: call, errCode: -1, errMsg: "\(error)")
			self.invokeListener(type: ListenerType.onSnapshotComplete, params: ["errCode": -1, "errMsg": "\(error)", "path": nil]);
		}
      });
      result(nil);
    }
  }
    
	/**
	* 静音/取消静音本地的音频
	*/
	public func muteLocalAudio(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let mute = CommonUtils.getParamByKey(call: call, result: result, param: "mute") as? Bool {
			trtcCloud.muteLocalAudio(mute);
			result(nil);
		}
	}
	
	/**
	* 暂停/恢复推送本地的视频数据
	*/
	public func muteLocalVideo(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let mute = CommonUtils.getParamByKey(call: call, result: result, param: "mute") as? Bool {
			trtcCloud.muteLocalVideo(mute);
			result(nil);
		}
	}
	
	/**
	* 设置音频路由
	*/
	public func setAudioRoute(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let route = CommonUtils.getParamByKey(call: call, result: result, param: "route") as? Int {
			txDeviceManager.setAudioRoute(TXAudioRoute(rawValue: route)!);
			result(nil);
		}
	}
	
	/**
	* 启用音量大小提示
	*/
	public func enableAudioVolumeEvaluation(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let intervalMs = CommonUtils.getParamByKey(call: call, result: result, param: "intervalMs") as? UInt {
			trtcCloud.enableAudioVolumeEvaluation(intervalMs);
			result(nil);
		}
	}
	
	/**
	* 开始录音
	*/
	public func startAudioRecording(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let param = CommonUtils.getParamByKey(call: call, result: result, param: "param") as? String {
			let dict = JsonUtil.getDictionaryFromJSONString(jsonString: param);
			let data = TRTCAudioRecordingParams();
			data.filePath = dict["filePath"] as! String;
			result(trtcCloud.startAudioRecording(data));
		}
	}
	
	/**
	* 停止录音
	*/
	public func stopAudioRecording(call: FlutterMethodCall, result: @escaping FlutterResult) {
		trtcCloud.stopAudioRecording();
		result(nil);
	}
	
	/**
	* 设置通话时使用的系统音量类型
	*/
	public func setSystemVolumeType(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let type = CommonUtils.getParamByKey(call: call, result: result, param: "type") as? Int {
			txDeviceManager.setSystemVolumeType(TXSystemVolumeType(rawValue: type)!);
			result(nil);
		}
	}
	
	/**
	* 创建 TRTCCloud 单例
	*/
	public func sharedInstance(call: FlutterMethodCall, result: @escaping FlutterResult) {
		trtcCloud = TRTCCloud.sharedInstance();
		txAudioEffectManager = trtcCloud.getAudioEffectManager();
		txBeautyManager = trtcCloud.getBeautyManager();
		txDeviceManager = trtcCloud.getDeviceManager();
		TRTCCloud.sharedInstance()?.delegate = self;
		result(nil);
	}
	
	/**
	* 销毁 TRTCCloud 单例
	*/
	public func destroySharedInstance(call: FlutterMethodCall, result: @escaping FlutterResult) {
		TRTCCloud.destroySharedIntance();
		result(nil);
	}
	
	/**
	* 获取美颜管理对象
	*/
	public func getBeautyManager(call: FlutterMethodCall, result: @escaping FlutterResult) {
		txBeautyManager = trtcCloud.getBeautyManager();
		result(nil);
	}

	/**
	* 获取设备管理对象
	*/
	public func getDeviceManager(call: FlutterMethodCall, result: @escaping FlutterResult) {
		txDeviceManager = trtcCloud.getDeviceManager();
		result(nil);
	}

    /**
     * 设置水印
     */
    public func setWatermark(call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let imageUrl = CommonUtils.getParamByKey(call: call, result: result, param: "imageUrl") as? String,
           let streamType = CommonUtils.getParamByKey(call: call, result: result, param: "streamType") as? Int,
           let x = CommonUtils.getParamByKey(call: call, result: result, param: "x") as? String,
           let y = CommonUtils.getParamByKey(call: call, result: result, param: "y") as? String,
           let width = CommonUtils.getParamByKey(call: call, result: result, param: "width") as? String,
		   let type = CommonUtils.getParamByKey(call: call, result: result, param: "type") as? String {
            
			let fx = CGFloat.init(Float.init(x)!)
            let fy = CGFloat.init(Float.init(y)!)
            let fwidth = CGFloat.init(Float.init(width)!)
			let rect = CGRect(x: fx, y: fy, width: fwidth, height: fwidth)

			if type == "local" {
				trtcCloud.setWatermark(UIImage.init(contentsOfFile: imageUrl), streamType: TRTCVideoStreamType.init(rawValue: streamType)!, rect: rect)
			} else {
				let queue = DispatchQueue(label: "setWatermark")
				queue.async {
					let url: NSURL = NSURL(string: imageUrl)!
					let data: NSData = NSData(contentsOf: url as URL)!
					let img = UIImage(data: data as Data, scale: 1)!
					self.trtcCloud.setWatermark(img, streamType: TRTCVideoStreamType.init(rawValue: streamType)!, rect: rect)
				}
			}

            result(nil);
        }
    }
	
	/**
	* 获取音效管理对象
	*/
	public func getAudioEffectManager(call: FlutterMethodCall, result: @escaping FlutterResult) {
		txAudioEffectManager = trtcCloud.getAudioEffectManager();
		result(nil);
	}
	
	/**
     * 发送自定义消息给房间内所有用户
     */
    public func sendCustomCmdMsg(call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let cmdID = CommonUtils.getParamByKey(call: call, result: result, param: "cmdID") as? Int,
           let dataStr = CommonUtils.getParamByKey(call: call, result: result, param: "data") as? String,
           let reliable = CommonUtils.getParamByKey(call: call, result: result, param: "reliable") as? Bool,
           let ordered = CommonUtils.getParamByKey(call: call, result: result, param: "ordered") as? Bool {
            let nsdata = dataStr.data(using: String.Encoding.utf8);
            result(trtcCloud.sendCustomCmdMsg(cmdID, data: nsdata, reliable: reliable, ordered: ordered));
        }
    }

    /**
     * 将小数据量的自定义数据嵌入视频帧中
     */
    public func sendSEIMsg(call: FlutterMethodCall, result: @escaping FlutterResult) {
        if  let dataStr = CommonUtils.getParamByKey(call: call, result: result, param: "data") as? String,
            let repeatCount = CommonUtils.getParamByKey(call: call, result: result, param: "repeatCount") as? Int32 {
            let nsdata = dataStr.data(using: String.Encoding.utf8);
            result(trtcCloud.sendSEIMsg(nsdata, repeatCount: repeatCount));
        }
    }

	/**
	* 开始进行网络测速（视频通话期间请勿测试，以免影响通话质量）
	*/
	public func startSpeedTest(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let sdkAppId = CommonUtils.getParamByKey(call: call, result: result, param: "sdkAppId") as? UInt32,
			 let userId = CommonUtils.getParamByKey(call: call, result: result, param: "userId") as? String,
			 let userSig = CommonUtils.getParamByKey(call: call, result: result, param: "userSig") as? String {
			trtcCloud.startSpeedTest(sdkAppId, userId: userId, userSig: userSig, completion: {
				(result, completedCount, totalCount) -> Void in
				print(result as Any, completedCount, totalCount)
				
			});
			result(nil);
			
		}
	}
	
	/**
	* 停止服务器测速
	*/
	public func stopSpeedTest(call: FlutterMethodCall, result: @escaping FlutterResult) {
		trtcCloud.stopSpeedTest();
		result(nil);
	}
	
	/**
	* 获取 SDK 版本信息
	*/
	public func getSDKVersion(call: FlutterMethodCall, result: @escaping FlutterResult) {
		result(TRTCCloud.getSDKVersion());
	}
	
	/**
	* 切换摄像头
	*/
	public func switchCamera(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let isFrontCamera = CommonUtils.getParamByKey(call: call, result: result, param: "isFrontCamera") as? Bool {
			txDeviceManager.switchCamera(isFrontCamera);
			result(nil);
		}
		
	}

	/**
	* 查询是前置摄像头
	*/
	public func isFrontCamera(call: FlutterMethodCall, result: @escaping FlutterResult) {
		result(txDeviceManager.isFrontCamera());
	}

	/**
	* 查询摄像头最大缩放率
	*/
	public func getCameraZoomMaxRatio(call: FlutterMethodCall, result: @escaping FlutterResult) {
		result(Int(txDeviceManager.getCameraZoomMaxRatio()));
	}
	
	/**
	* 设置摄像头缩放率
	*/
	public func setCameraZoomRatio(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let value = CommonUtils.getParamByKey(call: call, result: result, param: "value") as? String {
			let ret = txDeviceManager.setCameraZoomRatio(CGFloat(Int(Float(value)!)));
			result(ret);
		}
	}

	/**
	* 设置摄像头缩放率
	*/
	public func enableCameraAutoFocus(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Bool {
			result(txDeviceManager.enableCameraAutoFocus(enable));
		}
	}
	
	/**
	* 设置摄像头闪光灯，开启后置摄像头才有效果
	*/
	public func enableCameraTorch(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Bool {
			txDeviceManager.enableCameraTorch(enable);
			result(nil);
		}
	}
	
	/**
	* 设置对焦位置
	*/
	public func setCameraFocusPosition(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let x = CommonUtils.getParamByKey(call: call, result: result, param: "x") as? Int,
		   let y = CommonUtils.getParamByKey(call: call, result: result, param: "y") as? Int {
			txDeviceManager.setCameraFocusPosition(CGPoint(x: CGFloat(x), y: CGFloat(y)));
			result(nil);
		}
	}
	
	/**
	* 查询摄像头是否自动对焦
	*/
	public func isAutoFocusEnabled(call: FlutterMethodCall, result: @escaping FlutterResult) {
		result(txDeviceManager.isAutoFocusEnabled());
	}
		
	/**
	* 设置美颜（磨皮）算法
	* TXBeautyStyleSmooth, TXBeautyStyleNature, TXBeautyStylePitu
	*/
	public func setBeautyStyle(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let beautyStyle = CommonUtils.getParamByKey(call: call, result: result, param: "beautyStyle") as? Int {
			txBeautyManager.setBeautyStyle(TXBeautyStyle(rawValue: beautyStyle)!);
			result(nil);
		}
	}
	
	/**
	* 设置美颜级别
	*/
	public func setBeautyLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let beautyLevel = CommonUtils.getParamByKey(call: call, result: result, param: "beautyLevel") as? Int {
			txBeautyManager.setBeautyLevel(Float(beautyLevel));
			result(nil);
		}
	}
	
	/**
	* 设置美白级别
	*/
	public func setWhitenessLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let whitenessLevel = CommonUtils.getParamByKey(call: call, result: result, param: "whitenessLevel") as? Int {
			txBeautyManager.setWhitenessLevel(Float(whitenessLevel));
			result(nil);
		}
	}
	
	/**
	* 开启清晰度增强
	*/
	public func enableSharpnessEnhancement(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Bool {
			txBeautyManager.enableSharpnessEnhancement(enable);
			result(nil);
		}
	}
	
	/**
	* 设置红润级别
	*/
	public func setRuddyLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let ruddyLevel = CommonUtils.getParamByKey(call: call, result: result, param: "ruddyLevel") as? Float {
			txBeautyManager.setRuddyLevel(ruddyLevel);
			result(nil);
		}
	}
	
	/**
	* 设置指定素材滤镜特效
	* image 指定素材，即颜色查找表图片。必须使用 png 格式
	*/
	public func setFilter(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let imageUrl = CommonUtils.getParamByKey(call: call, result: result, param: "imageUrl") as? String,
			let type = CommonUtils.getParamByKey(call: call, result: result, param: "type") as? String {
			
			if type == "local" {
				let img = UIImage(contentsOfFile: imageUrl)!;
				txBeautyManager.setFilter(img);
			} else {
				let queue = DispatchQueue(label: "setFilter")
				queue.async {
					let url: NSURL = NSURL(string: imageUrl)!
					let data: NSData = NSData(contentsOf: url as URL)!
					let img = UIImage(data: data as Data, scale: 1)!
					self.txBeautyManager.setFilter(img);
				}
			}

			result(nil);
		}
	}
	
	/**
	* 设置滤镜浓度
	*/
	public func setFilterStrength(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let strength = CommonUtils.getParamByKey(call: call, result: result, param: "strength") as? String {
			txBeautyManager.setFilterStrength(Float(strength)!);
			result(nil);
		}
	}

	/**
	* TODO: 设置大眼级别，该接口仅在 企业版 SDK 中生效
	*/
	public func setEyeScaleLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setEyeScaleLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置瘦脸级别，该接口仅在 企业版 SDK 中生效
	*/
	public func setFaceSlimLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setFaceSlimLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置 V 脸级别，该接口仅在 企业版 SDK 中生效
	*/
	public func setFaceVLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setFaceVLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置下巴拉伸或收缩，该接口仅在 企业版 SDK 中生效
	*/
	public func setChinLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setChinLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置短脸级别，该接口仅在 企业版 SDK 中生效
	*/
	public func setFaceShortLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setFaceShortLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置瘦鼻级别，该接口仅在 企业版 SDK 中生效
	*/
	public func setNoseSlimLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setNoseSlimLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO：设置亮眼 ，该接口仅在 企业版 SDK 中生效
	*/
	public func setEyeLightenLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setEyeLightenLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO：设置白牙 ，该接口仅在 企业版 SDK 中生效
	*/
	public func setToothWhitenLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setToothWhitenLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置祛皱 ，该接口仅在 企业版 SDK 中生效
	*/
	public func setWrinkleRemoveLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setWrinkleRemoveLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置祛眼袋 ，该接口仅在 企业版 SDK 中生效
	*/
	public func setPounchRemoveLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setPounchRemoveLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置法令纹 ，该接口仅在 企业版 SDK 中生效
	*/
	public func setSmileLinesRemoveLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setSmileLinesRemoveLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置发际线 ，该接口仅在 企业版 SDK 中生效
	*/
	public func setForeheadLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setForeheadLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置眼距 ，该接口仅在 企业版 SDK 中生效
	*/
	public func setEyeDistanceLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setEyeDistanceLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置眼角 ，该接口仅在 企业版 SDK 中生效
	*/
	public func setEyeAngleLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setEyeAngleLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置嘴型 ，该接口仅在 企业版 SDK 中生效
	*/
	public func setMouthShapeLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setMouthShapeLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置鼻翼 ，该接口仅在 企业版 SDK 中生效
	*/
	public func setNoseWingLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setNoseWingLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置鼻子位置 ，该接口仅在 企业版 SDK 中生效
	*/
	public func setNosePositionLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setNosePositionLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置嘴唇厚度 ，该接口仅在 企业版 SDK 中生效
	*/
	public func setLipsThicknessLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setLipsThicknessLevel(enable);
			result(nil);
		}
	}
	
	/**
	* TODO: 设置脸型，该接口仅在 企业版 SDK 中生效
	*/
	public func setFaceBeautyLevel(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Float {
			txBeautyManager.setFaceBeautyLevel(enable);
			result(nil);
		}
	}
		
	/**
	* 开始播放背景音乐
	*/
	public func startPlayMusic(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let musicParam = CommonUtils.getParamByKey(call: call, result: result, param: "musicParam") as? String {
			let musicParam = JsonUtil.getDictionaryFromJSONString(jsonString: musicParam);
			let param = TXAudioMusicParam();

			param.id = musicParam["id"] as! Int32
			param.path = musicParam["path"] as! String
			param.loopCount = musicParam["loopCount"] as! Int
			param.publish = musicParam["publish"] as! Bool
			param.isShortFile = musicParam["isShortFile"] as! Bool
			param.startTimeMS = musicParam["startTimeMS"] as! Int
			param.endTimeMS = musicParam["endTimeMS"] as! Int
			
			txAudioEffectManager.startPlayMusic(param, onStart: {
				(errCode) -> Void in

				self.invokeListener(type: ListenerType.onMusicObserverStart, params: ["id": param.id, "errCode": errCode]);

			}, onProgress: {
				(progressMs, durationMs) -> Void in				

				self.invokeListener(type: ListenerType.onMusicObserverPlayProgress, params: ["id": param.id, "curPtsMS": progressMs, "durationMS": durationMs]);

			}, onComplete: {
				(errCode) -> Void in

				self.invokeListener(type: ListenerType.onMusicObserverComplete, params: ["id": param.id, "errCode": errCode]);
			});
			result(nil);
		}
	}
	
	/**
	* 开启耳返
	*/
	public func enableVoiceEarMonitor(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let enable = CommonUtils.getParamByKey(call: call, result: result, param: "enable") as? Bool {
			txAudioEffectManager.enableVoiceEarMonitor(enable);
			result(nil);
		}
	}
	
	/**
	* 设置耳返音量
	*/
	public func setVoiceEarMonitorVolume(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let volume = CommonUtils.getParamByKey(call: call, result: result, param: "volume") as? Int {
			txAudioEffectManager.setVoiceEarMonitorVolume(volume);
			result(nil);
		}
	}
	
	/**
	* 设置人声的混响效果（KTV、小房间、大会堂、低沉、洪亮...）
	*/
	public func setVoiceReverbType(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let type = CommonUtils.getParamByKey(call: call, result: result, param: "type") as? Int {
			txAudioEffectManager.setVoiceReverbType(TXVoiceReverbType(rawValue: type)!);
			result(nil);
		}
	}
	
	/**
	* 设置人声的变声特效（萝莉、大叔、重金属、外国人...）
	*/
	public func setVoiceChangerType(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let type = CommonUtils.getParamByKey(call: call, result: result, param: "type") as? Int {
			txAudioEffectManager.setVoiceChangerType(TXVoiceChangeType(rawValue: type)!);
			result(nil);
		}
	}
	
	/**
	* 设置麦克风采集人声的音量
	*/
	public func setVoiceCaptureVolume(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let volume = CommonUtils.getParamByKey(call: call, result: result, param: "volume") as? Int {
			txAudioEffectManager.setVoiceVolume(volume);
			result(nil);
		}
	}
	
	/**
	* 停止播放背景音乐
	*/
	public func stopPlayMusic(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let id = CommonUtils.getParamByKey(call: call, result: result, param: "id") as? Int32 {
			txAudioEffectManager.stopPlayMusic(id);
			result(nil);
		}
	}
	
	/**
	* 暂停播放背景音乐
	*/
	public func pausePlayMusic(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let id = CommonUtils.getParamByKey(call: call, result: result, param: "id") as? Int32 {
			txAudioEffectManager.pausePlayMusic(id);
			result(nil);
		}
	}
	
	/**
	* 恢复播放背景音乐
	*/
	public func resumePlayMusic(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let id = CommonUtils.getParamByKey(call: call, result: result, param: "id") as? Int32 {
			txAudioEffectManager.resumePlayMusic(id);
			result(nil);
		}
	}
	
	/**
	* 设置背景音乐的远端音量大小，即主播可以通过此接口设置远端观众能听到的背景音乐的音量大小。
	*/
	public func setMusicPublishVolume(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let id = CommonUtils.getParamByKey(call: call, result: result, param: "id") as? Int32,
			 let volume = CommonUtils.getParamByKey(call: call, result: result, param: "volume") as? Int {
			txAudioEffectManager.setMusicPublishVolume(id, volume: volume);
			result(nil);
		}
	}
	
	/**
	* 设置背景音乐的本地音量大小，即主播可以通过此接口设置主播自己本地的背景音乐的音量大小。
	* volume 音量大小，100为正常音量，取值范围为0 - 100；默认值：100
	*/
	public func setMusicPlayoutVolume(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let id = CommonUtils.getParamByKey(call: call, result: result, param: "id") as? Int32,
			 let volume = CommonUtils.getParamByKey(call: call, result: result, param: "volume") as? Int {
			txAudioEffectManager.setMusicPlayoutVolume(id, volume: volume);
			result(nil);
		}
	}
	
	/**
	* 设置全局背景音乐的本地和远端音量的大小
	*/
	public func setAllMusicVolume(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let volume = CommonUtils.getParamByKey(call: call, result: result, param: "volume") as? Int {
			txAudioEffectManager.setAllMusicVolume(volume);
			result(nil);
		}
	}
	
	/**
	* 调整背景音乐的音调高低
	* pitch	音调，默认值是0.0f，范围是：[-1 ~ 1] 之间的浮点数
	*/
	public func setMusicPitch(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let volume = CommonUtils.getParamByKey(call: call, result: result, param: "volume") as? Int32,
			 let pitch = Double.init((CommonUtils.getParamByKey(call: call, result: result, param: "pitch") as? String)!) {
			txAudioEffectManager.setMusicPitch(volume, pitch: pitch);
			result(nil);
		}
	}
	
	/**
	* 调整背景音乐的变速效果
	* speedRate	速度，默认值是1.0f，范围是：[0.5 ~ 2] 之间的浮点数
	*/
	public func setMusicSpeedRate(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let id = CommonUtils.getParamByKey(call: call, result: result, param: "id") as? Int32,
			 let speedRate = Double.init((CommonUtils.getParamByKey(call: call, result: result, param: "speedRate") as? String)!) {
			txAudioEffectManager.setMusicSpeedRate(id, speedRate: speedRate);
			result(nil);
		}
	}
	
	/**
	* 获取背景音乐当前的播放进度（单位：毫秒）
	*/
	public func getMusicCurrentPosInMS(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let id = CommonUtils.getParamByKey(call: call, result: result, param: "id") as? Int32 {
			let ms = txAudioEffectManager.getMusicCurrentPos(inMS: id);
			result(ms);
		}
	}
	
	/**
	* 设置背景音乐的播放进度（单位：毫秒）
	*/
	public func seekMusicToPosInMS(call: FlutterMethodCall, result: @escaping FlutterResult) {
		if let id = CommonUtils.getParamByKey(call: call, result: result, param: "id") as? Int32,
			 let pts = CommonUtils.getParamByKey(call: call, result: result, param: "pts") as? Int {
			txAudioEffectManager.seekMusicToPos(inMS: id, pts: pts);
			result(nil);
		}
	}
	
	/**
	* 获取景音乐文件的总时长（单位：毫秒）
	*/
	public func getMusicDurationInMS(call: FlutterMethodCall, result: @escaping FlutterResult) {
		let path = CommonUtils.getParamByKeyCanBeNull(call: call, result: result, param: "path") as? String
		let res = txAudioEffectManager.getMusicDuration(inMS: path != nil ? path! : "");
		result(res);
	}
	
	/**
	* 监听回调
	*/
	private func invokeListener(type: ListenerType, params: Any?) {
		var resultParams: [String: Any] = [:];
		resultParams["type"] = type;
		if let p = params {
			resultParams["params"] = p;
		}
		TencentTRTCCloud.channel!.invokeMethod(TencentTRTCCloud.LISTENER_FUNC_NAME, arguments: JsonUtil.toJson(resultParams));
	}
	
	/**
	* 错误回调，表示 SDK 不可恢复的错误，一定要监听并分情况给用户适当的界面提示。
	*/
	public func onError(_ errCode: TXLiteAVError, errMsg: String?, extInfo: [AnyHashable: Any]?) {
        self.invokeListener(type: ListenerType.onError, params: ["errCode": errCode.rawValue, "errMsg": errMsg as Any, "extraInfo": extInfo as Any]);
	}
	
	/**
	* 警告回调，用于告知您一些非严重性问题，例如出现了卡顿或者可恢复的解码失败。
	*/
	public func onWarning(_ warningCode: TXLiteAVWarning, warningMsg: String?, extInfo: [AnyHashable: Any]?) {
        self.invokeListener(type: ListenerType.onWarning, params: ["warningCode": warningCode.rawValue, "warningMsg": warningMsg as Any, "extraInfo": extInfo as Any]);
	}
	
	/**
	* 已加入房间的回调
	*/
	public func onEnterRoom(_ result: Int) {
		self.invokeListener(type: ListenerType.onEnterRoom, params: result);
	}
	
	/**
	* 离开房间的事件回调
	*/
	public func onExitRoom(_ reason: Int) {
		self.invokeListener(type: ListenerType.onExitRoom, params: reason);
	}
	
	/**
	* 切换角色的事件回调
	*/
	public func onSwitchRole(_ errCode: TXLiteAVError, errMsg: String?) {
        self.invokeListener(type: ListenerType.onSwitchRole, params: ["errCode": errCode.rawValue, "errMsg": errMsg as Any]);
	}
	
	/**
	* 请求跨房通话（主播 PK）的结果回调
	*/
	public func onConnectOtherRoom(_ userId: String, errCode: TXLiteAVError, errMsg: String?) {
        self.invokeListener(type: ListenerType.onConnectOtherRoom, params: ["userId": userId, "errCode": errCode.rawValue, "errMsg": errMsg as Any]);
	}

	/**
	* 切换房间 (switchRoom) 的结果回调
	*/
	public func onSwitchRoom(_ errCode: TXLiteAVError, errMsg: String?) {
        self.invokeListener(type: ListenerType.onSwitchRoom, params: ["errCode": errCode.rawValue, "errMsg": errMsg as Any]);
	}
	
	/**
	* 结束跨房通话（主播 PK）的结果回调
	*/
	public func onDisconnectOtherRoom(_ errCode: TXLiteAVError, errMsg: String?) {
        self.invokeListener(type: ListenerType.onDisConnectOtherRoom, params: ["errCode": errCode.rawValue, "errMsg": errMsg as Any]);
	}
	
	/**
	* 有用户加入当前房间
	*/
	public func onRemoteUserEnterRoom(_ userId: String) {
		self.invokeListener(type: ListenerType.onRemoteUserEnterRoom, params: userId);
	}
	
	/**
	* 有用户离开当前房间
	*/
	public func onRemoteUserLeaveRoom(_ userId: String, reason: Int) {
		self.invokeListener(type: ListenerType.onRemoteUserLeaveRoom, params: ["userId": userId, "reason": reason]);
	}
	
	/**
	* 有用户上传视频数据
	*/
	public func onUserVideoAvailable(_ userId: String, available: Bool) {
		self.invokeListener(type: ListenerType.onUserVideoAvailable, params: ["userId": userId, "available": available]);
	}
	
	/**
	* 远端用户是否存在可播放的辅路画面（一般用于屏幕分享）
	*/
	public func onUserSubStreamAvailable(_ userId: String, available: Bool) {
		self.invokeListener(type: ListenerType.onUserSubStreamAvailable, params: ["userId": userId, "available": available]);
	}
	
	/**
	* 远端用户是否存在可播放的音频数据
	*/
	public func onUserAudioAvailable(_ userId: String, available: Bool) {
		self.invokeListener(type: ListenerType.onUserAudioAvailable, params: ["userId": userId, "available": available]);
	}
	
	/**
	* 开始渲染本地或远程用户的首帧画面。
	*/
	public func onFirstVideoFrame(_ userId: String, streamType: TRTCVideoStreamType, width: Int32, height: Int32) {
		self.invokeListener(type: ListenerType.onFirstVideoFrame, params: ["userId": userId, "streamType": streamType.rawValue, "width": width, "height": height]);
	}
	
	/**
	* 开始播放远程用户的首帧音频（本地声音暂不通知）
	*/
	public func onFirstAudioFrame(_ userId: String) {
		self.invokeListener(type: ListenerType.onFirstAudioFrame, params: userId);
	}
	
	/**
	* 首帧本地视频数据已经被送出
	*/
	public func onSendFirstLocalVideoFrame(_ streamType: TRTCVideoStreamType) {
		self.invokeListener(type: ListenerType.onSendFirstLocalVideoFrame, params: streamType.rawValue);
	}
	
	/**
	* 首帧本地音频数据已经被送出
	*/
	public func onSendFirstLocalAudioFrame() {
		self.invokeListener(type: ListenerType.onSendFirstLocalAudioFrame, params: nil);
	}
	
	/**
	* 网络质量：该回调每2秒触发一次，统计当前网络的上行和下行质量。
	*/
	public func onNetworkQuality(_ localQuality: TRTCQualityInfo, remoteQuality: [TRTCQualityInfo]) {
		var array: [Any] = [];

		for item in remoteQuality {
            array.append(["userId": item.userId as Any, "quality": item.quality.rawValue]);
		}
		
		self.invokeListener(type: ListenerType.onNetworkQuality, params: [
			"localQuality": [
                "userId": localQuality.userId as Any,
				"quality": localQuality.quality.rawValue
			],
			"remoteQuality": array
		]);
	}
	
	/**
	* 技术指标统计回调
	*/
	public func onStatistics(_ statistics: TRTCStatistics) {
		self.invokeListener(type: ListenerType.onStatistics, params: [
            "upLoss": statistics.upLoss,
            "downLoss": statistics.downLoss,
            "appCpu": statistics.appCpu,
            "systemCpu": statistics.systemCpu,
            "rtt": statistics.rtt,
            "receivedBytes": statistics.receivedBytes,
            "sendBytes": statistics.sentBytes
        ]);
	}
	
	/**
	* SDK 跟服务器的连接断开
	*/
	public func onConnectionLost() {
		self.invokeListener(type: ListenerType.onConnectionLost, params: nil);
	}
	
	/**
	* SDK 尝试重新连接到服务器
	*/
	public func onTryToReconnect() {
		self.invokeListener(type: ListenerType.onTryToReconnect, params: nil);
	}
	
	/**
	* SDK 跟服务器的连接恢复
	*/
	public func onConnectionRecovery() {
		self.invokeListener(type: ListenerType.onConnectionRecovery, params: nil);
	}
	
	/**
	* 摄像头准备就绪
	*/
	public func onCameraDidReady() {
		self.invokeListener(type: ListenerType.onCameraDidReady, params: nil);
	}
	
	/**
	* 麦克风准备就绪
	*/
	public func onMicDidReady() {
		self.invokeListener(type: ListenerType.onMicDidReady, params: nil);
	}
	
	/**
	* 音频路由发生变化（仅 iOS），音频路由即声音由哪里输出（扬声器或听筒）
	*/
	public func onAudioRouteChanged(_ route: TRTCAudioRoute, from fromRoute: TRTCAudioRoute) {
		self.invokeListener(type: ListenerType.onAudioRouteChanged, params: ["newRoute": route.rawValue, "oldRoute": fromRoute.rawValue]);
	}
	
	
	/**
	* 用于提示音量大小的回调，包括每个 userId 的音量和远端总音量
	*/
	public func onUserVoiceVolume(_ userVolumes: [TRTCVolumeInfo], totalVolume: Int) {
		var userVolumeArray: [Any] = [];
		for item in userVolumes {
			userVolumeArray.append([
                "userId": item.userId as Any,
				"volume": item.volume
			]);
		}
		
		self.invokeListener(type: ListenerType.onUserVoiceVolume, params: ["userVolumes": userVolumeArray, "totalVolume": totalVolume]);
	}
	
    /**
     * 收到自定义消息回调
     */
    public func onRecvCustomCmdMsgUserId(_ userId: String, cmdID: Int, seq: UInt32, message: Data) {
        self.invokeListener(type: ListenerType.onRecvCustomCmdMsg, params: [
            "userId": userId,
            "cmdID": cmdID,
            "seq": seq,
            "message": String(data: message, encoding: String.Encoding.utf8) as String? as Any
        ]);
    }

    /**
     * 自定义消息丢失回调
     */
    public func onMissCustomCmdMsgUserId(_ userId: String, cmdID: Int, errCode: Int, missed: Int) {
        self.invokeListener(type: ListenerType.onMissCustomCmdMsg, params: [
            "userId": userId,
            "cmdID": cmdID,
            "errCode": errCode,
            "missed": missed
        ]);
    }

    /**
     * 收到 SEI 消息的回调
     */
    public func onRecvSEIMsg(_ userId: String, message: Data) {
        self.invokeListener(type: ListenerType.onRecvSEIMsg, params: [
            "userId": userId,
            "message": String(data: message, encoding: String.Encoding.utf8) as String? as Any
        ]);
    }
	
	/**
	* 开始向腾讯云的直播 CDN 推流的回调，对应于 TRTCCloud 中的 startPublishing() 接口
	*/
	public func onStartPublishing(_ err: Int32, errMsg: String) {
		self.invokeListener(type: ListenerType.onStartPublishing, params: [
			"errCode": err,
			"errMsg": errMsg
		]);
	}
	
	/**
	* 停止向腾讯云的直播 CDN 推流的回调，对应于 TRTCCloud 中的 stopPublishing() 接口
	*/
	public func onStopPublishing(_ err: Int32, errMsg: String) {
		self.invokeListener(type: ListenerType.onStopPublishing, params: [
			"errCode": err,
			"errMsg": errMsg
		]);
	}
	
	/**
	* 设置云端的混流转码参数的回调，对应于 TRTCCloud 中的 setMixTranscodingConfig() 接口。
	*/
	public func onSetMixTranscodingConfig(_ err: Int32, errMsg: String) {
		self.invokeListener(type: ListenerType.onSetMixTranscodingConfig, params: [
			"errCode": err,
			"errMsg": errMsg
		]);
	}
	
	/**
	* 播放音效结束回调
	*/
	public func onAudioEffectFinished(_ effectId: Int32, code: Int32) {
		self.invokeListener(type: ListenerType.onAudioEffectFinished, params: [
			"effectId": effectId,
			"errCode": code
		]);
	}
}
